import pyxel
from pyxel.ui import Widget


class OctaveBar(Widget):
    def __init__(self, parent, x, y):
        super().__init__(parent, x, y, 4, 123)

        self.add_event_handler("mouse_down", self.__on_mouse_down)
        self.add_event_handler("mouse_drag", self.__on_mouse_drag)
        self.add_event_handler("mouse_hover", self.__on_mouse_hover)
        self.add_event_handler("draw", self.__on_draw)

    def __on_mouse_down(self, key, x, y):
        if key != pyxel.MOUSE_LEFT_BUTTON:
            return

        x -= self.x
        y -= self.y

        self.parent.octave = min(max(3 - ((y - 12) // 24), 0), 3)

    def __on_mouse_drag(self, key, x, y, dx, dy):
        self.__on_mouse_down(key, x, y)

    def __on_mouse_hover(self, x, y):
        self.parent.help_message = "OCTAVE:PAGEUP/PAGEDOWN"

    def __on_draw(self):
        pyxel.rect(self.x, self.y, self.width, self.height, 6)

        x = self.x + 1
        y = self.y + 1 + (3 - self.parent.octave) * 24
        pyxel.rect(x, y, 2, 47, 13)



import pyxel
from pyxel.ui import Widget

from .constants import (
    TILEMAP_IMAGE_HEIGHT,
    TILEMAP_IMAGE_WIDTH,
    TILEMAP_IMAGE_X,
    TILEMAP_IMAGE_Y,
)


class TilemapPanel(Widget):
    def __init__(self, parent):
        super().__init__(parent, 157, 16, 66, 65)

        self.add_event_handler("mouse_down", self.__on_mouse_down)
        self.add_event_handler("mouse_drag", self.__on_mouse_drag)
        self.add_event_handler("mouse_hover", self.__on_mouse_hover)
        self.add_event_handler("draw", self.__on_draw)

    def _screen_to_view(self, x, y):
        x = min(max((x - self.x - 1) // 2, 0), 30) * 8
        y = min(max((y - self.y - 1) // 2, 0), 30) * 8
        return x, y

    def __on_mouse_down(self, key, x, y):
        if key == pyxel.MOUSE_LEFT_BUTTON:
            self.parent.drawing_x, self.parent.drawing_y = self._screen_to_view(x, y)

    def __on_mouse_drag(self, key, x, y, dx, dy):
        if key == pyxel.MOUSE_LEFT_BUTTON:
            self.__on_mouse_down(key, x, y)

    def __on_mouse_hover(self, x, y):
        x, y = self._screen_to_view(x, y)
        self.parent.help_message = "TARGET:CURSOR ({},{})".format(x, y)

    def __on_draw(self):
        self.draw_panel(self.x, self.y, self.width, self.height)

        pyxel.blt(
            self.x + 1,
            self.y + 1,
            3,
            TILEMAP_IMAGE_X,
            TILEMAP_IMAGE_Y,
            TILEMAP_IMAGE_WIDTH,
            TILEMAP_IMAGE_HEIGHT,
        )

        pyxel.clip(self.x + 1, self.y + 1, self.width - 2, self.height - 2)

        x = self.x + self.parent.drawing_x // 4 + 1
        y = self.y + self.parent.drawing_y // 4 + 1
        w = 4
        h = 4

        pyxel.rectb(x, y, w, h, 7)
        pyxel.rectb(x - 1, y - 1, w + 2, h + 2, 0)

        pyxel.clip()



from pyxel import _get_constant_number, _get_constant_string

APP_WIDTH = 240
APP_HEIGHT = 180

EDITOR_IMAGE_X = 0
EDITOR_IMAGE_Y = 32
EDITOR_IMAGE_NAME = "editor_220x160.png"

TILEMAP_IMAGE_X = 0
TILEMAP_IMAGE_Y = 192
TILEMAP_IMAGE_WIDTH = 64
TILEMAP_IMAGE_HEIGHT = 63

TOOL_SELECT = 0
TOOL_PENCIL = 1
TOOL_RECTB = 2
TOOL_RECT = 3
TOOL_CIRCB = 4
TOOL_CIRC = 5
TOOL_BUCKET = 6

MAX_SOUND_LENGTH = 48
MAX_MUSIC_LENGTH = 32

IMAGE_BANK_COUNT = _get_constant_number("IMAGE_BANK_COUNT")
TILEMAP_BANK_COUNT = _get_constant_number("TILEMAP_BANK_COUNT")
SOUND_BANK_COUNT = _get_constant_number("SOUND_BANK_COUNT")
MUSIC_BANK_COUNT = _get_constant_number("MUSIC_BANK_COUNT")
MUSIC_CHANNEL_COUNT = _get_constant_number("MUSIC_CHANNEL_COUNT")
FONT_WIDTH = _get_constant_number("FONT_WIDTH")
FONT_HEIGHT = _get_constant_number("FONT_HEIGHT")

RESOURCE_FILE_EXTENSION = _get_constant_string("RESOURCE_FILE_EXTENSION")



import os.path

import pyxel
from pyxel.ui import ColorPicker, NumberPicker, RadioButton

from .constants import EDITOR_IMAGE_X, EDITOR_IMAGE_Y, IMAGE_BANK_COUNT, TOOL_PENCIL
from .drawing_panel import DrawingPanel
from .editor import Editor
from .image_panel import ImagePanel


class ImageEditor(Editor):
    def __init__(self, parent):
        super().__init__(parent)

        self._drawing_panel = DrawingPanel(self, is_tilemap_mode=False)
        self._image_panel = ImagePanel(self, is_tilemap_mode=False)
        self._color_picker = ColorPicker(self, 11, 156, 7, with_shadow=False)
        self._tool_button = RadioButton(
            self, 81, 161, 3, EDITOR_IMAGE_X + 63, EDITOR_IMAGE_Y, 7, TOOL_PENCIL
        )
        self._image_picker = NumberPicker(self, 192, 161, 0, IMAGE_BANK_COUNT - 2, 0)

        self.add_event_handler("undo", self.__on_undo)
        self.add_event_handler("redo", self.__on_redo)
        self.add_event_handler("drop", self.__on_drop)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)
        self.add_tool_button_help(self._tool_button)
        self.add_number_picker_help(self._image_picker)

    @property
    def color(self):
        return self._color_picker.value

    @color.setter
    def color(self, value):
        self._color_picker.value = int(value)

    @property
    def tool(self):
        return self._tool_button.value

    @tool.setter
    def tool(self, value):
        self._tool_button.value = value

    @property
    def image(self):
        return self._image_picker.value

    @property
    def drawing_x(self):
        return self._drawing_panel.viewport_x

    @drawing_x.setter
    def drawing_x(self, value):
        self._drawing_panel.viewport_x = value

    @property
    def drawing_y(self):
        return self._drawing_panel.viewport_y

    @drawing_y.setter
    def drawing_y(self, value):
        self._drawing_panel.viewport_y = value

    def __on_undo(self, data):
        img = data["image"]
        x, y = data["pos"]
        dest = pyxel.image(img).data[y : y + 16, x : x + 16]
        dest[:, :] = data["before"]

        self.drawing_x = x
        self.drawing_y = y
        self.parent.image = img

    def __on_redo(self, data):
        img = data["image"]
        x, y = data["pos"]
        dest = pyxel.image(img).data[y : y + 16, x : x + 16]
        dest[:, :] = data["after"]

        self.drawing_x = x
        self.drawing_y = y
        self.parent.image = img

    def __on_drop(self, filename):
        _, ext = os.path.splitext(filename)

        if ext.lower() == ".png":
            pyxel.image(self.image).load(0, 0, filename)
            return

    def __on_update(self):
        self.check_tool_button_shortcuts()

    def __on_draw(self):
        self.draw_panel(11, 156, 136, 17)
        self.draw_panel(157, 156, 72, 17)
        pyxel.text(170, 162, "IMAGE", 6)



import pyxel


class App:
    def __init__(self):
        pyxel.init(255, 81, caption="Pyxel Color Palette")
        pyxel.run(self.update, self.draw)

    def update(self):
        if pyxel.btnp(pyxel.KEY_Q):
            pyxel.quit()

    def draw(self):
        pyxel.cls(0)

        for i in range(16):
            self.draw_palette(2 + (i % 4) * 64, 4 + (i // 4) * 20, i)

    def draw_palette(self, x, y, col):
        col_val = pyxel.DEFAULT_PALETTE[col]
        hex_col = "#{:06X}".format(col_val)
        rgb_col = "{},{},{}".format(
            col_val >> 16, (col_val >> 8) & 0xFF, col_val & 0xFF
        )

        pyxel.rect(x, y, 13, 13, col)
        pyxel.text(x + 16, y + 1, hex_col, 7)
        pyxel.text(x + 16, y + 8, rgb_col, 7)
        pyxel.text(
            x + 5 - (col // 10) * 2, y + 4, "{}".format(col), 7 if col < 6 else 0
        )

        if col == 0:
            pyxel.rectb(x, y, 13, 13, 5)


App()



import pyxel
from pyxel.ui import Widget
from pyxel.ui.constants import WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME

from .constants import EDITOR_IMAGE_X, EDITOR_IMAGE_Y, MAX_SOUND_LENGTH


class PianoRoll(Widget):
    def __init__(self, parent):
        super().__init__(parent, 30, 25, 193, 123)

        self._press_x = 0
        self._press_y = 0

        self.add_event_handler("mouse_down", self.__on_mouse_down)
        self.add_event_handler("mouse_up", self.__on_mouse_up)
        self.add_event_handler("mouse_drag", self.__on_mouse_drag)
        self.add_event_handler("mouse_click", self.__on_mouse_click)
        self.add_event_handler("mouse_hover", self.__on_mouse_hover)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)

    def _screen_to_view(self, x, y):
        x = min(max((x - self.x - 1) // 4, 0), MAX_SOUND_LENGTH - 1)
        y = min(max(59 - (y - self.y - 1) // 2, -1), 59)
        return x, y

    def __on_mouse_down(self, key, x, y):
        if key != pyxel.MOUSE_LEFT_BUTTON or self.parent.is_playing:
            return

        x, y = self._screen_to_view(x, y)

        self._press_x = x
        self._press_y = y

        self.parent.field_cursor.move(x, 0)

    def __on_mouse_up(self, key, x, y):
        pass

    def __on_mouse_drag(self, key, x, y, dx, dy):
        if key != pyxel.MOUSE_LEFT_BUTTON or self.parent.is_playing:
            return

        x, y = self._screen_to_view(x, y)

        if x > self._press_x:
            x1 = self._press_x
            y1 = self._press_y
            x2 = x
            y2 = y
        elif x < self._press_x:
            x1 = x
            y1 = y
            x2 = self._press_x
            y2 = self._press_y
        else:
            return

        self.parent.add_pre_history(x, 0)

        data = self.parent.field_cursor.data
        padding_length = self._press_x + 1 - len(data)
        if padding_length > 0:
            data.extend([-1] * padding_length)

        self._press_x = x
        self._press_y = y

        self.parent.field_cursor.move(x, 0)

        dx = x2 - x1
        dy = y2 - y1

        for i in range(dx + 1):
            value = round(y1 + (dy / dx) * i)

            if x1 + i >= len(data):
                data.append(value)
            else:
                data[x1 + i] = value

        self.parent.add_post_history(x, 0)

    def __on_mouse_click(self, key, x, y):
        if key != pyxel.MOUSE_LEFT_BUTTON or self.parent.is_playing:
            return

        x, y = self._screen_to_view(x, y)

        self.parent.field_cursor.move(x, 0)

        data = self.parent.field_cursor.data

        self.parent.add_pre_history(x, 0)

        padding_length = x + 1 - len(data)
        if padding_length > 0:
            data.extend([-1] * padding_length)

        data[x] = y

        self.parent.add_post_history(x, 0)

    def __on_mouse_hover(self, x, y):
        self.parent.help_message = "NOTE:CLICK/PIANO_KEY+ENTER/BS/DEL"

    def __on_update(self):
        cursor_y = self.parent.field_cursor.y

        if cursor_y > 0 or self.parent.is_playing:
            return

        if (
            pyxel.btnp(pyxel.KEY_ENTER, WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME)
            or pyxel.btnp(pyxel.KEY_KP_ENTER, WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME)
        ) and self.parent.keyboard_note is not None:
            self.parent.field_cursor.insert(self.parent.keyboard_note)

    def __on_draw(self):
        pyxel.rect(self.x, self.y, self.width, self.height, 6)

        if self.parent.is_playing:
            x = (self.parent.play_pos % 100) * 4 + 31
            pyxel.rect(x, 25, 3, 123, 2)
        else:
            if self.parent.field_cursor.y == 0:
                x = self.parent.field_cursor.x * 4 + 31
                pyxel.rect(x, 25, 3, 123, 1)

        pyxel.blt(self.x, self.y, 3, EDITOR_IMAGE_X, EDITOR_IMAGE_Y + 7, 193, 72, 6)
        pyxel.blt(
            self.x, self.y + 72, 3, EDITOR_IMAGE_X, EDITOR_IMAGE_Y + 7, 193, 51, 6
        )

        for i, note in enumerate(self.parent.get_data(0)):
            x = i * 4 + 31
            y = 143 - note * 2
            pyxel.rect(x, y, 3, 3, 8 if note >= 0 else 12)



import pyxel

from .constants import INPUT_FIELD_COLOR, INPUT_TEXT_COLOR
from .text_button import TextButton
from .widget import Widget


class NumberPicker(Widget):
    """
    Events:
        __on_change(value)
    """

    def __init__(self, parent, x, y, min_value, max_value, value, **kwargs):
        self._number_len = max(len(str(min_value)), len(str(max_value)))
        width = self._number_len * 4 + 21
        height = 7
        super().__init__(parent, x, y, width, height, **kwargs)

        self._min_value = min_value
        self._max_value = max_value
        self._value = None

        self.dec_button = TextButton(self, x, y, "-")
        self.inc_button = TextButton(self, x + width - 7, y, "+")

        self.add_event_handler("enabled", self.__on_enabled)
        self.add_event_handler("disabled", self.__on_disabled)
        self.add_event_handler("draw", self.__on_draw)
        self.dec_button.add_event_handler("press", self.__on_dec_button_press)
        self.dec_button.add_event_handler("repeat", self.__on_dec_button_press)
        self.inc_button.add_event_handler("press", self.__on_inc_button_press)
        self.inc_button.add_event_handler("repeat", self.__on_inc_button_press)

        self.value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if self._value != value:
            self._value = value
            self.call_event_handler("change", value)

            self.dec_button.is_enabled = self._value != self._min_value
            self.inc_button.is_enabled = self._value != self._max_value

    def __on_enabled(self):
        self.dec_button.is_enabled = self._value != self._min_value
        self.inc_button.is_enabled = self._value != self._max_value

    def __on_disabled(self):
        self.dec_button.is_enabled = False
        self.inc_button.is_enabled = False

    def __on_draw(self):
        pyxel.rect(self.x + 9, self.y, self.width - 18, self.height, INPUT_FIELD_COLOR)

        pyxel.text(
            self.x + 11,
            self.y + 1,
            ("{:>" + str(self._number_len) + "}").format(self._value),
            INPUT_TEXT_COLOR,
        )

    def __on_dec_button_press(self):
        offset = 10 if pyxel.btn(pyxel.KEY_SHIFT) else 1
        self.value = max(self._value - offset, self._min_value)

    def __on_inc_button_press(self):
        offset = 10 if pyxel.btn(pyxel.KEY_SHIFT) else 1
        self.value = min(self._value + offset, self._max_value)



import glob
import os
import shutil


def install():
    src_dir = os.path.dirname(__file__)
    dest_dir = os.path.join(os.getcwd(), "pyxel_examples")

    print("Install Pyxel examples to {} ...".format(dest_dir))

    shutil.rmtree(dest_dir, ignore_errors=True)
    os.makedirs(os.path.join(dest_dir, "assets"))

    patterns = ["[0-9]*.py", "assets/*.pyxel", "assets/*.png", "assets/*.gif"]

    for pattern in patterns:
        srcs = glob.glob(os.path.join(src_dir, pattern))

        for src in srcs:
            relpath = os.path.relpath(src, src_dir)
            dest = os.path.join(dest_dir, relpath)

            print("    {}".format(relpath))
            shutil.copyfile(src, dest)


if __name__ == "__main__":
    install()



import pyxel
from pyxel.ui import Widget
from pyxel.ui.constants import WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME

from .constants import EDITOR_IMAGE_X, EDITOR_IMAGE_Y, MAX_SOUND_LENGTH

tone_key_table = [pyxel.KEY_T, pyxel.KEY_S, pyxel.KEY_P, pyxel.KEY_N]
effect_key_table = [pyxel.KEY_N, pyxel.KEY_S, pyxel.KEY_V, pyxel.KEY_F]


class SoundField(Widget):
    def __init__(self, parent):
        super().__init__(parent, 30, 149, 193, 23)

        self.add_event_handler("mouse_down", self.__on_mouse_down)
        self.add_event_handler("mouse_hover", self.__on_mouse_hover)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)

    def _screen_to_view(self, x, y):
        x = min(max((x - self.x - 1) // 4, 0), MAX_SOUND_LENGTH - 1)
        y = min(max((y - self.y) // 8, 0), 2)
        return x, y

    def __on_mouse_down(self, key, x, y):
        if key != pyxel.MOUSE_LEFT_BUTTON or self.parent.is_playing:
            return

        x, y = self._screen_to_view(x, y)
        self.parent.field_cursor.move(x, y + 1)

    def __on_mouse_hover(self, x, y):
        x, y = self._screen_to_view(x, y)
        if y == 0:
            self.parent.help_message = "TONE:T/S/P/N/BS/DEL"
        elif y == 1:
            self.parent.help_message = "VOLUME:0-7/BS/DEL"
        elif y == 2:
            self.parent.help_message = "EFFECT:N/S/V/F/BS/DEL"

    def __on_update(self):
        cursor_y = self.parent.field_cursor.y

        if (
            cursor_y < 1
            or self.parent.is_playing
            or pyxel.btn(pyxel.KEY_SHIFT)
            or pyxel.btn(pyxel.KEY_CONTROL)
            or pyxel.btn(pyxel.KEY_ALT)
        ):
            return

        value = None
        if cursor_y == 1:
            for i in range(4):
                if pyxel.btnp(tone_key_table[i], WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME):
                    value = i
                    break
        elif cursor_y == 2:
            for i in range(8):
                if pyxel.btnp(
                    pyxel.KEY_0 + i, WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME
                ) or pyxel.btnp(
                    pyxel.KEY_KP_0 + i, WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME
                ):
                    value = i
                    break
        elif cursor_y == 3:
            for i in range(4):
                if pyxel.btnp(
                    effect_key_table[i], WIDGET_HOLD_TIME, WIDGET_REPEAT_TIME
                ):
                    value = i
                    break

        if value is None:
            return

        self.parent.field_cursor.insert(value)

    def __on_draw(self):
        pyxel.text(self.x - 13, self.y + 1, "TON", 6)
        pyxel.text(self.x - 13, self.y + 9, "VOL", 6)
        pyxel.text(self.x - 13, self.y + 17, "EFX", 6)
        pyxel.blt(self.x, self.y, 3, EDITOR_IMAGE_X, EDITOR_IMAGE_Y + 79, 193, 23)

        data_str = []
        data_str.append("".join(["TSPN"[v] for v in self.parent.get_data(1)]))
        data_str.append("".join([str(v) for v in self.parent.get_data(2)]))
        data_str.append("".join(["NSVF"[v] for v in self.parent.get_data(3)]))

        for i in range(3):
            pyxel.text(31, 150 + i * 8, data_str[i], 1)

        cursor_y = self.parent.field_cursor.y
        cursor_x = self.parent.field_cursor.x

        if self.parent.is_playing or cursor_y == 0:
            return

        x = cursor_x * 4 + 31
        y = cursor_y * 8 + 142

        pyxel.rect(x, y - 1, 3, 6, 1)

        if cursor_x < len(data_str[cursor_y - 1]):
            pyxel.text(x, y, data_str[cursor_y - 1][cursor_x], 7)



import pyxel
from pyxel.ui import Widget

from .constants import EDITOR_IMAGE_X, EDITOR_IMAGE_Y

key_table = [
    pyxel.KEY_Z,
    pyxel.KEY_S,
    pyxel.KEY_X,
    pyxel.KEY_D,
    pyxel.KEY_C,
    pyxel.KEY_V,
    pyxel.KEY_G,
    pyxel.KEY_B,
    pyxel.KEY_H,
    pyxel.KEY_N,
    pyxel.KEY_J,
    pyxel.KEY_M,
    pyxel.KEY_Q,
    pyxel.KEY_2,
    pyxel.KEY_W,
    pyxel.KEY_3,
    pyxel.KEY_E,
    pyxel.KEY_R,
    pyxel.KEY_5,
    pyxel.KEY_T,
    pyxel.KEY_6,
    pyxel.KEY_Y,
    pyxel.KEY_7,
    pyxel.KEY_U,
]


class PianoKeyboard(Widget):
    def __init__(self, parent):
        super().__init__(parent, 17, 25, 12, 123)

        self._sound = pyxel.sound(64, system=True)
        self._sound.set("g2", "p", "3", "n", 30)
        self._mouse_note = None
        self.note = None
        self._tone = 0

        self.add_event_handler("mouse_down", self.__on_mouse_down)
        self.add_event_handler("mouse_up", self.__on_mouse_up)
        self.add_event_handler("mouse_drag", self.__on_mouse_drag)
        self.add_event_handler("mouse_hover", self.__on_mouse_hover)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)

    def _screen_to_note(self, x, y):
        x -= self.x
        y -= self.y

        octave = (4 - y // 24) * 12
        y %= 24

        if octave > 59:
            return 59
        if octave < 0:
            return -1

        if x <= 6:
            if 2 <= y <= 4:
                return octave + 10
            elif 6 <= y <= 8:
                return octave + 8
            elif 10 <= y <= 12:
                return octave + 6
            elif 16 <= y <= 18:
                return octave + 3
            elif 20 <= y <= 22:
                return octave + 1

        if y <= 2:
            return octave + 11
        elif y <= 6:
            return octave + 9
        elif y <= 10:
            return octave + 7
        elif y <= 13:
            return octave + 5
        elif y <= 16:
            return octave + 4
        elif y <= 20:
            return octave + 2
        else:
            return octave

    def __on_mouse_down(self, key, x, y):
        if key == pyxel.MOUSE_LEFT_BUTTON:
            self._mouse_note = self._screen_to_note(x, y)

    def __on_mouse_up(self, key, x, y):
        self._mouse_note = None

    def __on_mouse_drag(self, key, x, y, dx, dy):
        if key == pyxel.MOUSE_LEFT_BUTTON:
            self._mouse_note = self._screen_to_note(x, y)

    def __on_mouse_hover(self, x, y):
        self.parent.help_message = "PLAY:Z/S/X..Q/2/W..A TONE:1"

    def __on_update(self):
        if (
            self.parent.field_cursor.y > 0
            or self.parent.is_playing
            or pyxel.btn(pyxel.KEY_SHIFT)
            or pyxel.btn(pyxel.KEY_CONTROL)
            or pyxel.btn(pyxel.KEY_ALT)
        ):
            return

        if pyxel.btnp(pyxel.KEY_1):
            self._tone = (self._tone + 1) % 4

        self.note = self._mouse_note
        for i, key in enumerate(key_table):
            if pyxel.btn(key):
                self.note = self.parent.octave * 12 + i
                break

        if pyxel.btn(pyxel.KEY_A):
            self.note = -1

        if self.note is not None:
            self._sound.note[0] = self.note
            self._sound.tone[0] = self._tone
            pyxel.play(1, 64)
        else:
            pyxel.stop(1)

    def __on_draw(self):
        pyxel.blt(self.x, self.y, 3, EDITOR_IMAGE_X + 208, EDITOR_IMAGE_Y, 12, 123, 6)

        data = self.parent.get_data(0)

        if (
            self.parent.is_playing
            and not data
            or not self.parent.is_playing
            and self.note is None
        ):
            return

        note = data[self.parent.play_pos] if self.parent.is_playing else self.note
        x = self.x
        y = self.y + (59 - note) * 2
        key = note % 12

        if note == -1:
            pyxel.rect(x, y + 1, 12, 2, 12)
        elif key == 0 or key == 5:
            pyxel.rect(x, y + 1, 7, 1, 14)
            pyxel.rect(x + 7, y, 5, 2, 14)
        elif key == 4 or key == 11:
            pyxel.rect(x, y + 1, 7, 1, 14)
            pyxel.rect(x + 7, y + 1, 5, 2, 14)
        elif key == 2 or key == 7 or key == 9:
            pyxel.rect(x, y + 1, 7, 1, 14)
            pyxel.rect(x + 7, y, 5, 3, 14)
        else:
            pyxel.rect(x, y + 1, 6, 1, 14)



import setuptools

from pyxel import VERSION

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyxel",
    version=VERSION,
    description="A retro game engine for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kitao/pyxel",
    author="Takashi Kitao",
    author_email="takashi.kitao@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Games/Entertainment",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Multimedia :: Sound/Audio",
    ],
    packages=[
        "pyxel",
        "pyxel.ui",
        "pyxel.core",
        "pyxel.core.bin.macos",
        "pyxel.core.bin.win32",
        "pyxel.core.bin.win64",
        "pyxel.core.bin.linux",
        "pyxel.editor",
        "pyxel.editor.assets",
        "pyxel.examples",
        "pyxel.examples.assets",
    ],
    package_data={"": ["*.pyxres", "*.png", "*.gif", "*.dylib", "*.dll", "*.so"]},
    install_requires=["numpy"],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "pyxeleditor=pyxel.editor:run",
            "install_pyxel_examples=pyxel.examples:install",
        ]
    },
)



import os
import sys

import pyxel
from pyxel.editor.app import App


def run():
    arg = sys.argv[1] if len(sys.argv) >= 2 else ""

    if arg.startswith("-"):
        if arg == "-v" or arg == "--version":
            print("Pyxel Editor {}".format(pyxel.VERSION))
        else:
            print("Usage: pyxeleditor [option] [pyxel_resource_file]")
            print("Options:")
            print(" -h, --help     This help text")
            print(" -v, --version  Show version number and quit")
    else:
        filename = arg or os.path.join(os.getcwd(), "my_resource")
        App(filename)


if __name__ == "__main__":
    sys.argv[1:] = ["assets/test_resource"]
    run()



import pyxel
from pyxel.ui import NumberPicker, RadioButton

from .constants import (
    EDITOR_IMAGE_X,
    EDITOR_IMAGE_Y,
    IMAGE_BANK_COUNT,
    TILEMAP_BANK_COUNT,
    TILEMAP_IMAGE_X,
    TILEMAP_IMAGE_Y,
    TOOL_PENCIL,
)
from .drawing_panel import DrawingPanel
from .editor import Editor
from .image_panel import ImagePanel
from .tilemap_panel import TilemapPanel


class TileMapEditor(Editor):
    def __init__(self, parent):
        super().__init__(parent)

        self._drawing_panel = DrawingPanel(self, is_tilemap_mode=True)
        self._tilemap_panel = TilemapPanel(self)
        self._image_panel = ImagePanel(self, is_tilemap_mode=True)
        self._tilemap_picker = NumberPicker(self, 48, 161, 0, TILEMAP_BANK_COUNT - 1, 0)
        self._tool_button = RadioButton(
            self, 81, 161, 3, EDITOR_IMAGE_X + 63, EDITOR_IMAGE_Y, 7, TOOL_PENCIL
        )
        self._image_picker = NumberPicker(
            self,
            192,
            161,
            0,
            IMAGE_BANK_COUNT - 2,
            pyxel.tilemap(self._tilemap_picker.value).refimg,
        )

        self.add_event_handler("undo", self.__on_undo)
        self.add_event_handler("redo", self.__on_redo)
        self.add_event_handler("update", self.__on_update)
        self.add_event_handler("draw", self.__on_draw)
        self._tilemap_picker.add_event_handler(
            "change", self.__on_tilemap_picker_change
        )
        self._image_picker.add_event_handler("change", self.__on_image_picker_change)
        self.add_number_picker_help(self._tilemap_picker)
        self.add_number_picker_help(self._image_picker)
        self.add_tool_button_help(self._tool_button)

    @property
    def tilemap(self):
        return self._tilemap_picker.value

    @tilemap.setter
    def tilemap(self, value):
        self._tilemap_button.value = value

    @property
    def color(self):
        return self._image_panel.focused_tiles

    @color.setter
    def color(self, value):
        self._image_panel.set_focus(value % 32 * 8, value // 32 * 8)

    @property
    def tool(self):
        return self._tool_button.value

    @tool.setter
    def tool(self, value):
        self._tool_button.value = value

    @property
    def image(self):
        return self._image_picker.value

    @image.setter
    def image(self, value):
        self._image_button.value = value

    @property
    def drawing_x(self):
        return self._drawing_panel.viewport_x

    @drawing_x.setter
    def drawing_x(self, value):
        self._drawing_panel.viewport_x = value

    @property
    def drawing_y(self):
        return self._drawing_panel.viewport_y

    @drawing_y.setter
    def drawing_y(self, value):
        self._drawing_panel.viewport_y = value

    def __on_undo(self, data):
        tm = data["tilemap"]
        x, y = data["pos"]
        dest = pyxel.tilemap(tm).data[y : y + 16, x : x + 16]
        dest[:, :] = data["before"]

        self._drawing_panel.viewport_x = x
        self._drawing_panel.viewport_y = y
        self._tilemap_picker.value = tm

    def __on_redo(self, data):
        tm = data["tilemap"]
        x, y = data["pos"]
        dest = pyxel.tilemap(tm).data[y : y + 16, x : x + 16]
        dest[:, :] = data["after"]

        self._drawing_panel.viewport_x = x
        self._drawing_panel.viewport_y = y
        self._tilemap_picker.value = tm

    def __on_update(self):
        start_y = pyxel.frame_count % 8 * 8
        for y in range(start_y, start_y + 8):
            for x in range(64):
                val = pyxel.tilemap(self.tilemap).data[y * 4 + 1, x * 4 + 1]
                col = pyxel.image(self.image).data[val // 32 * 8 + 3, val % 32 * 8 + 3]
                pyxel.image(3, system=True).data[
                    TILEMAP_IMAGE_Y + y, TILEMAP_IMAGE_X + x
                ] = col

        self.check_tool_button_shortcuts()

    def __on_draw(self):
        self.draw_panel(11, 156, 136, 17)
        self.draw_panel(157, 156, 72, 17)
        pyxel.text(18, 162, "TILEMAP", 6)
        pyxel.text(18, 162, "TILEMAP", 6)
        pyxel.text(170, 162, "IMAGE", 6)

    def __on_tilemap_picker_change(self, value):
        self._image_picker.value = pyxel.tilemap(value).refimg

    def __on_image_picker_change(self, value):
        pyxel.tilemap(self._tilemap_picker.value).refimg = value
